﻿using Fw.Utility.UnifyResponse;
using Hotline.Ai.Jths.OrderVisits;
using Hotline.Ai.Visit;
using Hotline.Orders;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using RestSharp;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Hotline.Ai.Jths
{
    public class AiVisitService : IAiVisitService
    {
        private readonly RestClient _client;
        private readonly string _baseUrl;
        private readonly string _appkey;
        private readonly string _serviceversion;
        private readonly string _sceneuid;
        private readonly string _ruleuid;
        private readonly string _visitFromNameKey;
        private readonly string _visitFromGenderKey;
        private readonly string _visitCreationTimeKey;
        private readonly string _visitOrderTitleKey;

        public AiVisitService(string baseUrl, string appkey, string serviceversion, string sceneuid, string ruleuid,string visitFromNameKey,string visitFromGenderKey,string visitCreationTimeKey,string visitOrderTitleKey)
        {
            _baseUrl = baseUrl;
            _appkey = appkey;
            _serviceversion = serviceversion;
            _sceneuid = sceneuid;
            _ruleuid = ruleuid;
            _visitFromNameKey = visitFromNameKey;
            _visitFromGenderKey = visitFromGenderKey;
            _visitCreationTimeKey = visitCreationTimeKey;
            _visitOrderTitleKey = visitOrderTitleKey;
            var options = new RestClientOptions(_baseUrl);
            _client = new RestClient(options);
        }

        public async Task<AiOrderVisit> CreateAiOrderVisitTask(AiOrderVisit aiOrderVisit, CancellationToken cancellationToken)
        {
            var requestData = new AiVisitServiceRequest()
            {
                BatchName = aiOrderVisit.Name,
                BatchStatus = 1,
                SceneUid = _sceneuid,
                Priority = 1,
                StartDate = aiOrderVisit.BeginTime.ToString("yyyy/MM/dd HH:mm:ss"),
                EndDate = aiOrderVisit.EndTime.ToString("yyyy/MM/dd HH:mm:ss"),
                FestivalBan = aiOrderVisit.FestivalBan,
                RuleType = aiOrderVisit.RuleType,
                RuleUid = _ruleuid,
            };
            var taskDataList = new List<TaskData>();
            foreach (var item in aiOrderVisit.AiOrderVisitDetails)
            {
                var taskData = new TaskData();
                taskData.CalledNumber = item.OuterNo;
                taskData.VariableList = new List<Variable>();
               
                if (item.Order.FromGender!= EGender.Unknown)
                {
                    if (!string.IsNullOrEmpty(item.Order.FromName))
                    {
                        //开发环境 :OC_SCENE_VAR_FIELD11
                        //生产环境：OC_SCENE_VAR_FIELD12
                        taskData.VariableList.Add(new Variable() { Code = _visitFromNameKey, Value = item.Order.FromName });
                    }
                    //开发环境: OC_SCENE_VAR_FIELD14
                    //生产环境：OC_SCENE_VAR_FIELD14
                    taskData.VariableList.Add(new Variable() { Code = _visitFromGenderKey, Value = item.Order.FromGender == EGender.Female ? "女士" : "先生" });
                }
                //开发环境:OC_SCENE_VAR_FIELD17
                //生产环境：OC_SCENE_VAR_FIELD19
                taskData.VariableList.Add(new Variable() { Code = _visitCreationTimeKey, Value = item.Order.CreationTime.ToString("yyyy年MM月dd日hh点mm分") });
                //开发环境:OC_SCENE_VAR_FIELD18
                //生产环境：OC_SCENE_VAR_FIELD20
                taskData.VariableList.Add(new Variable() { Code = _visitOrderTitleKey, Value = item.Order.Title });
                taskDataList.Add(taskData);
            }
            requestData.TaskDataList = taskDataList;
            var response = await ExecuteAsync<AiVisitServiceRequest, AiVisitServiceResponse>(_baseUrl + "/edas/batchTask",Method.Post, requestData,cancellationToken);
            if (response.Result.TaskInfoList!=null && !string.IsNullOrEmpty(response.Result.BatchUid))
            {
                //拼对象
                aiOrderVisit.BatchUid = response.Result.BatchUid;
                for (int i = 0; i < aiOrderVisit.AiOrderVisitDetails.Count; i++)
                {
                    var taskInfo = response.Result.TaskInfoList.FirstOrDefault(x => x.CalledNumber == aiOrderVisit.AiOrderVisitDetails[i].OuterNo);
                    if (taskInfo != null)
                    {
                        aiOrderVisit.AiOrderVisitDetails[i].AiOrderVisitState = Share.Enums.Ai.EAiOrderVisitState.NoStarted;
                        aiOrderVisit.AiOrderVisitDetails[i].TaskUid = taskInfo.TaskUid;
                    }
                    else
                    {
                        aiOrderVisit.AiOrderVisitDetails[i].AiOrderVisitState = Share.Enums.Ai.EAiOrderVisitState.LoseEfficacy;
                        //处理回访状态TODO
                    }
                }
            }
            return aiOrderVisit;
        }


        public async Task<AiVisitQueryData> QueryAiVisitTask(string batchId, string taskId, CancellationToken cancellationToken)
        {
            var dto = new AiVisitServiceQueryRequest() { BatchUid = batchId, TaskUid = taskId,PageNo=1, PageSize=100 };
            var response = await ExecuteAsync<AiVisitServiceQueryRequest, AiVisitServiceQueryResponse>(_baseUrl + "/edas/task/status/real", Method.Get, dto, cancellationToken);
            if (response!=null && response.Result.Result.Count>0)
            {
                return new AiVisitQueryData() { Uid = response.Result.Result[0].Uid, Status = response.Result.Result[0].Status };
            }
            return null;
        }


        public async Task<TResponse> ExecuteAsync<TRequest,TResponse>(string path, Method httpMethod, TRequest request,
            CancellationToken cancellationToken)
            where TRequest : class
        {
            var req = new RestRequest(path, httpMethod);
            req.AddHeader("content-type", "application/json");
            req.AddHeader("token", "");
            req.AddHeader("version", _serviceversion);
            req.AddHeader("appkey", _appkey);
            if (httpMethod is Method.Get)
            {
                req.AddObject(request);
            }
            else
            {
                req.AddJsonBody(request);
            }
            try
            {
                var response = await _client.ExecuteAsync<TResponse>(req, cancellationToken);
                return response.Data;
            }
            catch (Exception e)
            {
                throw new HttpRequestException($"智能回访平台错误，Error: {e.Message}");
            }
        }
    }
}
